register_calling_convention

Function register_calling_convention 

pub fn register_calling_convention<A, C, F>(
    arch: &A,
    name: &str,
    func: F,
) -> Ref<CoreCallingConvention>
Registers a new calling convention with the given architecture.

The convention object is constructed by func, which is given the CoreCallingConvention handle of the newly created convention. Implementors must store this handle and return it from their AsRef<CoreCallingConvention> implementation so that the trait’s layout methods can delegate to the core’s default behavior.

NOTE: This function should only be called within CorePluginInit.